package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * AutoTest is a base class that simplifies the writing of tests. To
 * write a simple test, you merely extend AutoTest and write a bunch
 * of public methods that take no arguments and that return a
 * boolean. When that class is run, each method will be invoked once
 * and a success reported if it returns true and failure if it returns
 * false.
 */

public abstract class AutoTest extends BaseTest implements Test {

  private Map       testMethods   = null;
  private Map       testClasses   = null;
  private Set       runningTests  = new HashSet();
  private String    lastMessage   = null;
  private Throwable lastException = null;

  public AutoTest() {
    setName(getClass().getName());
  }
  
  public void run() {
    Date bomb = timebomb();
    if (bomb.after(new Date())) {
      reportSuccess("*", "Timebomb explodes on " + bomb);
    } else {
      runTestClasses();
      runTestMethods();
      cleanup();
    }
  }

  protected void reset() {
    testMethods = null;
    testClasses = null;
  }

  public Date timebomb() { return new Date(); }

  private void runTestMethods() {
    Iterator i = testMethods().values().iterator();
    while (i.hasNext()) {
      SingleMethodTest t = (SingleMethodTest)i.next();
      if (!t.hasRun()) runTest(t);
    }
  }

  private void runTestClasses() {
    Iterator i = testClasses().values().iterator();
    while (i.hasNext()) {
      Test t = (Test)i.next();
      t.run();
    }
  }

  public int getNumberOfTestCases() {
    return testMethods().keySet().size() + countCasesIn(testClasses());
  }

  private int countCasesIn(Map map) {
    Iterator i = map.values().iterator();
    int count = 0;
    while (i.hasNext()) {
      count += ((Test)i.next()).getNumberOfTestCases();
    }
    return count;
  }

  /**
   * Test writers can call this method from within their test case
   * methods to require that another named test have passed. This makes
   * it easier to write atomic test cases since they can be written
   * assuming some other test case has run and passed.
   */
  protected void require(String name) {
    SingleMethodTest t = findTest(name);
    if (!t.hasRun()) runTest(t);
    if (!t.passed()) {
     throw new PreconditionNotMetException(
       "Required test '" + name + "' didn't pass."); 
    }
  }

  protected boolean success() {
    return success("ok.");
  }

  protected boolean success(String msg) {
    saveResultInfo(msg, null);
    return true;
  }

  protected boolean failure(String msg) {
    saveResultInfo(msg, null);
    return false;
  }

  protected boolean failure(String msg, Throwable t) {
    saveResultInfo(msg, t);
    return false;
  }

  protected boolean expect(boolean expected, boolean got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(byte expected, byte got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(char expected, char got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(double expected, double got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(float expected, float got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(int expected, int got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(long expected, long got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(short expected, short got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  protected boolean expect(Object expected, Object got) {
    if (expected == got) {
      return success();
    } else {
      return failure("Got " + got + "; expected " + expected);
    }
  }

  private void saveResultInfo(String msg, Throwable t) {
    lastMessage = msg;
    lastException = t;
  }

  String getLastMessage() {
    String tmp = lastMessage;
    lastMessage = null;
    return tmp; 
  }
  Throwable getLastException() {
    Throwable tmp = lastException;
    lastException = null;
    return tmp;
  }

  private void runTest(SingleMethodTest t) {
    try {
      enterTest(t);
      t.run();
    } finally {
      leaveTest(t);
    }
  }

  private void enterTest(SingleMethodTest t) {
    if (runningTests.contains(t)) {
      throw new TestAbortedException("Cycle detected in AutoTest.require()");
    }
    runningTests.add(t);
  }

  private void leaveTest(SingleMethodTest t) {
    runningTests.remove(t);
  }


  private Map testMethods() {
    if (testMethods == null) {
      testMethods = new HashMap();
      Method[] methods = getClass().getMethods();
      for (int i = 0; i < methods.length; i++) {
	Method m = methods[i];
	if (isTestMethod(m)) {
	  SingleMethodTest smt = new SingleMethodTest(m, this);
	  testMethods.put(smt.getMethodName(), smt);
	}
      }
    }
    return testMethods;
  }

  private Map testClasses() {
    if (testClasses == null) {
      testClasses = new HashMap();
      Class[] innerClasses = getClass().getClasses();
      for (int i = 0; i < innerClasses.length; i++) {
	Class clazz = innerClasses[i];
	if (clazz.getDeclaringClass() == getClass().getDeclaringClass()) continue;
	if (Test.class.isAssignableFrom(clazz) && 
	    !Modifier.isAbstract(clazz.getModifiers())) {
	  try {
	    Test t;
	    if (!Modifier.isStatic(clazz.getModifiers())) {
	      Constructor cons = clazz.getConstructor(
		new Class[] { getClass() });
	      t = (Test)cons.newInstance(new Object[] { this });

	    } else {
	      t = (Test)clazz.newInstance();
	    }
	    t.addTestListener(this);
	    testClasses.put(clazz.getName(), t);
	  } catch (InstantiationException ie) {
	    throw new TestAbortedException(
	      "Badly formed inner test class " + clazz, ie);
	  } catch (IllegalAccessException iae) {
	    throw new TestAbortedException(
	      "Badly formed inner test class " + clazz, iae);
	  } catch (NoSuchMethodException nsme) {
	    throw new TestAbortedException(
	      "Badly formed inner test class " + clazz, nsme);
	  } catch (InvocationTargetException ite) {
	    throw new TestAbortedException(
	      "Badly formed inner test class " + clazz, ite);
	  }

	}
      }
    }
    return testClasses;
  }

  private SingleMethodTest findTest(String name) {
    return (SingleMethodTest)testMethods().get(name);
  }
  
  private boolean isTestMethod(Method m) {
    return 
      m.getReturnType() == Boolean.TYPE &&
      m.getParameterTypes().length == 0 &&
      Modifier.isPublic(m.getModifiers()) &&
      m.getName().startsWith("test");
  }

}








