package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/**
 * Interface for tests. Each Test comprises a number of test cases
 * which are run in the Test's run() method. The results of the test
 * cases are reported back via the TestListener interface. Well
 * behaved Tests will be implemented so that a given Test, run with
 * the same parameters, will generate the same set of test cases. That
 * is, each test case should generate <i>some</i> TestEvent each time
 * the Test is run.
 *
 * @see TestListener
 */
public interface Test extends Runnable {
  
  // Constants used in reporting the results of individual test cases.

  /**
   * Test case succeeded -- ran and behaved as expected.
   */
  int SUCCESS = 1;

  /**
   * Test case failed -- ran but behaved improperly.
   */
  int FAILURE = 2;

  /**
   * Test case was skipped -- wasn't run at all.
   */
  int SKIPPED = 3;

  /**
   * Test case aborted -- started to run but for some reason couldn't
   * complete either successfully or unsuccessfully.
   */
  int ABORTED = 4;

  /**
   * Run the test. Each test will comprise many test cases, each of
   * which will report its results via the TestListener interface.
   */
  public void run();

  /**
   * Get the name of the test. Used for logging.
   */
  public String getName();

  /**
   * Get the number of test cases contained by this Test.
   */
  public int getNumberOfTestCases();

  /**
   * Add a TestListener. The listener will be notified of the
   * beginning of the test run and the result of each test case in the
   * run.
   */
  public void addTestListener(TestListener l);

  /**
   * Remove a TestListener.
   */
  public void removeTestListener(TestListener l);

  /**
   * Do any needed post-test cleanup.
   */
  public void cleanup();
}




