package com.javamonkey.utils;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.*;
import java.io.PrintStream;

/**
 * A class for handling command line options. Each option can be
 * either a boolean flag or a variable option that takes some
 * value. Options can be declared with a documentation string and a
 * CommandLineOption object can emit a usage message of all the
 * options it understands.
 *
 * The CommandLineOption class really provides three APIs:
 * 
 * <p><b>1. Option defining API</b>. These methods are used to
 * programatically configure a CommandLineOptions object with a set of
 * flag and variable options it understands. This configuration must
 * be done before the other two APIs will be useful.
 *
 * <p><b>2. Option parsing API</b>. This API comprises a single
 * method: <code>parseCommandLine()</code> which extracts the values
 * of the defined options (and the arguments left over) out of an
 * array of Strings, typically the <code>String[]</code> passed to a
 * <code>main()</code> method.
 *
 * <p><b>3. Value reading API</b>. These methods can be used after
 * <code>parseCommandLine()</code> has been called to get the values
 * of individual options as various sort of primitive Java types as
 * well as the arguments that were left over after
 * <code>parseCommandLine()</code> extracted the options and their
 * values.
 */

public class CommandLineOptions {

  private final static String OPT_INDICATOR = "-";

  private Map options = new HashMap();
  private List arguments = new ArrayList();
  private int maxNameLength = 0;

  // =================================================================
  // Option defining API.
  
  /**
   * Add a flag (boolean option) to the set of options this object
   * understands.
   *
   * @param name The name of the flag. This name, prepended with the
   * OPT_INDICATOR string, appearing in the command line sets the
   * value of this flag to be true.
   * @param documentation A human readable documentation string of
   * this flag.
   */
  public void addFlag(String name, String documentation) {
    addOption(new Flag(name, documentation));
  }

  /**
   * Add a variable option to the set of options this object
   * understands. Uses the default paramDoc "value".
   *
   * @param name The name of the flag. This name, prepended with the
   * OPT_INDICATOR string, appearing in the command line sets the
   * value of this flag to be true.
   * @param documentation A human readable documentation string of
   * this variable.
   */
  public void addVariable(String name, String documentation) {
    addVariable(name, documentation, "value");
  }

  /**
   * Add a variable option to the set of options this object
   * understands.
   *
   * @param name The name of the flag. This name, prepended with the
   * OPT_INDICATOR string, appearing in the command line sets the
   * value of this flag to be true.
   * @param documentation A human readable documentation string of
   * this variable.
   * @param paramDoc A string that describes the value that should be
   * passed to this variable. For example a variable that expects a
   * file name might use the String "file".
   */
  public void addVariable(String name, String documentation, String paramDoc) {
    addOption(new Variable(name, documentation, paramDoc));
  }

  // =================================================================
  // Option parsing API

  /**
   * Parse the the command line arguments contained in the given List.
   * 
   * @param argv The command line arguments. Typically this will be a
   * the <code>String[]</code> passed to a <code>main()</code> method.
   */
  public void parseCommandLine(String[] argv) {
    List argsLeft = new ArrayList();

    for (int i = 0; i < argv.length; ) {
      String s = argv[i];
      if (s.startsWith(OPT_INDICATOR)) {
	String name = s.substring(OPT_INDICATOR.length());
	i += getOption(name).parse(argv, i);
      } else {
	arguments.add(s);
	i++;
      }
    }
  }

  // =================================================================
  // Value reading API.

  /**
   * Return the arguments left after all the options (and their
   * values) have been removed.
   */
  public Iterator arguments() {
    return arguments.iterator();
  }

  public int numberOfArguments() {
    return arguments.size();
  }

  public String argument(int idx) {
    return (String)arguments.get(idx);
  }

  /**
   * Is the given option set. (If the option is a flag this is all you
   * can find out about it.)
   */
  public boolean isSet(String name) {
    return getOption(name).isSet();
  }
  
  /**
   * Return the value of the named option as a String.
   */
  public String asString(String name) {
    return getOption(name).asString();
  }

  /**
   * Return the value of the named option as a boolean.
   */
  public boolean asBoolean(String name) {
    return getOption(name).asBoolean();
  }

  /**
   * Return the value of the named option as a byte.
   */
  public byte asByte(String name) {
    return getOption(name).asByte();
  }

  /**
   * Return the value of the named option as a char.
   */
  public char asChar(String name) {
    return getOption(name).asChar();
  }

  /**
   * Return the value of the named option as a double.
   */
  public double asDouble(String name) {
    return getOption(name).asDouble();
  }

  /**
   * Return the value of the named option as a float.
   */
  public float asFloat(String name) {
    return getOption(name).asFloat();
  }

  /**
   * Return the value of the named option as a int.
   */
  public int asInt(String name) {
    return getOption(name).asInt();
  }

  /**
   * Return the value of the named option as a long.
   */
  public long asLong(String name) {
    return getOption(name).asLong();
  }

  /**
   * Return the value of the named option as a short.
   */
  public short asShort(String name) {
    return getOption(name).asShort();
  }
  
  
  // =================================================================
  // Usage API

  public void emitUsage(String command, String args) {
    PrintStream out = System.err;

    out.println("Usage: ");
    out.println();
    out.println("  " + command + " <opts> " + args);
    out.println();
    Iterator i = options.values().iterator();
    if (i.hasNext()) {
      out.println("where <opts> are any of:");
      out.println();
      while (i.hasNext()) {
	Option o = (Option)i.next();
	String name = paddedName(OPT_INDICATOR + o.getName());
	String doc = o.getDocumentation();
	out.println("  " + name + " " + doc);
      }
    }
  }

  private String paddedName(String name) {
    int len = OPT_INDICATOR.length() + maxNameLength;
    StringBuffer buf = new StringBuffer(len);
    buf.append(name);
    for (int i = name.length(); i < len; i++) {
      buf.append(" ");
    }
    return buf.toString();
  }
  

  // =================================================================
  // Private methods.

  private void addOption(Option opt) {
    options.put(opt.getName(), opt);
    maxNameLength = Math.max(maxNameLength, opt.getName().length());
  }

  private Option getOption(String name) {
    return (Option)options.get(name);
  }

  // =================================================================
  // Helper classes

  private abstract class Option {

    String name;
    String documentation;
    String value = null;

    Option(String name, String documentation) {
      this.name = name;
      this.documentation = documentation;
    }

    /**
     * Extract whatever information from the given argument list and
     * return the number of elements that were used.
     *
     * @param args The complete list of command line arguments.
     * @param index The index into the list of the option marker.
     * @return The number of elements of args used. (I.e. one for
     * boolean flags, two for variable options.)
     */
    abstract int parse(String[] args, int index);

    String getName() { return name; }
    String getDocumentation() { return documentation; }

    boolean isSet() { return value != null; }

    String asString() { return value; }
    boolean asBoolean() { return Boolean.valueOf(value).booleanValue(); }
    byte asByte() { return Byte.valueOf(value).byteValue(); }
    char asChar() { return value.charAt(0); }
    double asDouble() { return Double.valueOf(value).doubleValue(); }
    float asFloat() { return Float.valueOf(value).floatValue(); }
    int asInt() { return Integer.decode(value).intValue(); }
    long asLong() { return Long.decode(value).longValue(); }
    short asShort() { return Short.valueOf(value).shortValue(); }
    
  }

  private class Flag extends Option {

    Flag(String name, String documentation) {
      super(name, documentation);
    }

    int parse(String[] args, int index) {
      value = String.valueOf(true);
      return 1;
    }
  }

  private class Variable extends Option {
    
    private String paramDoc;

    Variable(String name, String documentation, String paramDoc) {
      super(name, documentation);
      this.paramDoc = paramDoc;
    }

    int parse(String[] args, int index) {
      value = (String)args[index + 1];
      return 2;
    }

    String getDocumentation() {
      return "<" + paramDoc + "> " + documentation;
    }
  }
 

  public static void main(String[] argv) {

    CommandLineOptions opts = new CommandLineOptions();
    opts.addFlag("x", "x is a test flag");
    opts.addFlag("y", "y is another test flag.");
    opts.addVariable("foo", "foo is a variable");
    opts.addVariable("bar", "bar is another variable", "frob");
    
    opts.parseCommandLine(argv);

    System.out.println("x " + (opts.isSet("x") ? "is" : "is not") + " set.");
    System.out.println("y " + (opts.isSet("y") ? "is" : "is not") + " set.");
    System.out.println("foo " + (opts.isSet("foo") ? "is" : "is not") + " set.");
    System.out.println("bar " + (opts.isSet("bar") ? "is" : "is not") + " set.");

    System.out.println("x.asBoolean() " + opts.asBoolean("x"));
    System.out.println("y.asBoolean() " + opts.asBoolean("y"));
    System.out.println("foo.asBoolean() " + opts.asBoolean("foo"));
    System.out.println("bar.asBoolean() " + opts.asBoolean("bar"));

    System.out.println("x.asString() " + opts.asString("x"));
    System.out.println("y.asString() " + opts.asString("y"));
    System.out.println("foo.asString() " + opts.asString("foo"));
    System.out.println("bar.asString() " + opts.asString("bar"));

    System.out.print("Actual arguments: ");
    Iterator i = opts.arguments();
    while (i.hasNext()) {
      System.out.print(i.next());
      if (i.hasNext()) System.out.print(" ");
    }
    System.out.println();

    opts.emitUsage("TEST", "<arguments>");
    
  }   
}
