package com.javamonkey.utils.iterators;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.*;

/**
 * An abstract base class for iterators that only know when they've
 * hit the end by walking past it. This class maintains a one-element
 * lookahead cache and properly implements the Iterator interface,
 * requiring only that the subclass provide a method
 * <code>nextObject</code> that coughs up the next object as
 * appropriate.
 *
 * @version $Id$
 */
public abstract class PeekingIterator implements Iterator {

  protected final static Object END = new Object();
  private final static Object EMPTY = new Object();

  private Object cache = EMPTY;
  private Object previous = EMPTY;
  
  /**
   * Implements <code>hasNext()</code> contract by checking look-ahead
   * cache and filling it by a call to nextObject() if necessary. If
   * the nextObject() returns END then there are no more objects in
   * the iteration.
   *
   * @see java.util.Iterator
   */
  public final boolean hasNext() {
    if (cache == EMPTY) cache = nextObject();
    return cache != END;
  }

  /**
   * Implements the <code>next()</code> contract by returning the
   * contents of the look-ahead cache. Throws NoSuchElementException
   * after the <code>nextObject()</code> return END.
   *
   * @see java.util.Iterator
   */
  public final Object next() {
    if (hasNext()) {
      previous = cache;
      cache = EMPTY;
      return previous;
    } else {
      throw new NoSuchElementException();
    }
  }

  /**
   * Implements the <code>remove()</code> contract of either throwing
   * an <code>UnsupportedOperationException</code> if
   * <code>supportsRemove()</code> returns false or by calling the
   * <code>doRemove()</code> method with a reference to the last
   * object returned by <code>next()</code>. If
   * <code>supportsRemove()</code> returns true, this method will also
   * <code>IllegalStateException</code>s if it is called out of turn.
   *
   * @see java.util.Iterator
   */
  public final void remove()
    throws IllegalStateException, UnsupportedOperationException
  {
    if (supportsRemove()) {
      if (previous == EMPTY) throw new IllegalStateException();
      Object tmp = previous;
      previous = EMPTY;
      doRemove(tmp);
    } else {
      throw new UnsupportedOperationException();
    }
  }


  /**
   * Method used by subclasses to return the next object. This method
   * should return PeekingIterator.END when there are no more elements
   * to be iterated over. It will not be called again after it returns
   * END.
   */
  protected abstract Object nextObject();

  /**
   * Method to be overridden by subclasses that want to implement the
   * <code>remove()</code> method. The object passed in is the last
   * object returned by <code>next()</code>. The default
   * implementation throws an UnsupportedOperationException().
   *
   * @see #supportsRemove()
   */
  protected void doRemove(Object returned) {
    throw new UnsupportedOperationException();
  }

  /**
   * Subclasses that support the <code>remove()</code> operation must
   * override this method return <code>true</code>. (This is necessary
   * so the implementation of <code>remove()</code> can properly
   * through <code>IllegalStateException</code>s for classes that do
   * implement <code>remove()</code> while throwing
   * <code>UnsupportedOperationException</code>s for those that do
   * not.) The default implementation returns false.
   *
   * @see #doRemove(Object)
   */
  protected boolean supportsRemove() { return false; }


}

